myname = 'max_vals_1-16_v2.R'

# leave false for 1-16 deg vers
do_BC_version = FALSE

source( '~/location.R' )
source( '../mod_descrip_v10.R' )
source( '../make_all_filenames_v4.R' )

source('~/src/mine/R/multi_page_png.R' )

seas = c('DJF', 'MAM', 'JJA', 'SON')

period_pat       = c('2006-2039', '2040-2069', '2070-2???' )
period_pat_label = c('2006-2039', '2040-2069', '2070-2100' )

test_no_plots = FALSE

#-------------------------------------------------------------------
# Since we deal with original model grids here, I make a common grid 
# and regrid all the orig model grids to it
#-------------------------------------------------------------------
xvals_com = seq( -125.5, -66.5, by=1 )
yvals_com = seq(   22.5,  53.5, by=1 )

#======================================================================================
get_delta = function( fname_fut, fname_hist, modd, ivar, regrid=FALSE, do_additive_precip=FALSE ) {

	if( test_no_plots ) {
		nxc = length( xvals_com )
		nyc = length( yvals_com )
		delta = rnorm( nxc*nyc )
		dim( delta ) = c( nxc, nyc )
		return( list(
			delta		= delta,
			xvals		= xvals_com,
			yvals		= yvals_com ))
		}

	ncid_hist = nc_open( fname_hist )
	var_hist  = ncid_hist$var[[1]]
	dat_hist  = ncvar_get( ncid_hist, var_hist  )
	ndims_hist = length(dim(dat_hist))

	ncid_fut  = nc_open( fname_fut )
	dat_fut   = ncvar_get( ncid_fut )
	ndims_fut = length(dim(dat_fut))

	if( (ndims_hist != 2) || (ndims_fut != 2))
		stop(paste("Error, ndims hist and fut must be 2"))

	xvals = var_hist$dim[[1]]$vals
	yvals = var_hist$dim[[2]]$vals

	if( max(xvals) > 181 )
		xvals = xvals - 360

	if( (ivar == IVARTYPE_PR) && (!do_additive_precip) ) 
		delta = (dat_fut - dat_hist)/dat_hist * 100.
	else	
		delta = dat_fut - dat_hist

	nc_close( ncid_hist )
	nc_close( ncid_fut  )

	#------------------------------
	# Implement regrid if requested
	#------------------------------
	if( regrid ) {
		mask_isgood_wanted = array( TRUE, dim=c(length(xvals_com), length(yvals_com)))
		delta = put_on_common_grid( xvals_com, yvals_com, mask_isgood_wanted, xvals, yvals, delta )
		xvals = xvals_com
		yvals = yvals_com
		}

	return( list(
		delta		= delta,
		xvals		= xvals,
		yvals		= yvals ))
}

#======================================================================================
add_mean = function( ff ) {

	ffm = mean( ff, na.rm=TRUE )

	rmse = sqrt( mean( ff**2, na.rm=TRUE ))

	s = paste( 'Mean:', mformatC('%.1f', ffm ), '   RMSE:', mformatC('%.1f', rmse) )

	par(font=2)
	text( -124, 50.5, s, adj=c(0,0), cex=1.8)
	par(font=1)
}

#======================================================================================
domodel_var = function( fname_all, modd, imod, ivar, is_last_mod=is_last_mod, ret_all=ret_all,
		do_additive_precip=FALSE, aspect='maxval' ) {

	nseas = length( seas )

	vv = modd$var[[ivar]]
	mm = modd$model[[imod]]

	modname = mm$modname_cmip5
	varname = vv$varname_cmip5

	basedir = '/nobackupp6/nexprojects/LOCAds2014'
	datdir_old_v8  = paste0( basedir, '/regrid_to_1x1_v8' )
	livdir  = paste0( basedir, '/Livneh/livneh_NAmerExt_15Oct2014' )

	#---------------------------------------
	# Compare historical monthly mean to obs
	#---------------------------------------
	fname_clim_liv = paste0( livdir, '/', 
		vv$varname_livneh, '.1950-2005.', vv$units_tag, '.crop.monthly.clim.nc' )

	idstype = IDSTYPE_16DEG
	iexpt = IEXPT_HISTORICAL
	iperiod = IPERIOD_TRAINING_PERIOD

	if( (! test_no_plots) && (! is_last_mod) ) {

		for( iseas in 1:nseas ) {

			print(paste("... doing season", iseas, '/', nseas, '.....'))
			print(paste("... doing season", iseas, '/', nseas, '.....'))
			fname_liv =  paste0( livdir, '/',
				vv$varname_livneh, '.1950-2005.', vv$units_tag, '.crop.all.', seas[iseas], '.', 
					aspect, '.nc' )

			#---------------------------
			# Open Livneh obs, read data
			#---------------------------
			print(paste("reading livneh file", fname_liv ))
			ncid_liv = nc_open( fname_liv )
			dat_liv = ncvar_get( ncid_liv )
			xvals_liv = ncid_liv$dim$lon$vals
			yvals_liv = ncid_liv$dim$lat$vals
			if( max(xvals_liv) > 181 ) xvals_liv = xvals_liv - 360
			nc_close( ncid_liv )

			fname_mod = paste0( '../', fname_all$dstype[[idstype]]$mod[[imod]]$var[[ivar]]$expt[[iexpt]]$period[[iperiod]]$ds )
			sub_str = paste0( 'srs.ds.postdsbc.all.', seas[iseas], '.', aspect, '.nc' )
			fname_mod = sub( 'srs.ds.nc', sub_str, fname_mod )

			#---------------------------
			# Open model file, read data
			#---------------------------
			print(paste("reading model file", fname_mod ))
			ncid_mod = nc_open( fname_mod )
			dat_mod = ncvar_get( ncid_mod )
			xvals_mod = ncid_mod$dim$lon$vals
			yvals_mod = ncid_mod$dim$lat$vals
			if( max(xvals_mod) > 181 ) xvals_mod = xvals_mod - 360
			nc_close( ncid_mod )

			#----------------
			# Set up for plot
			#----------------
			xlim = range( xvals_liv )
			ylim = range( yvals_liv )
			cfyo = 0.6
			cfh = 1.6

			mppng_source_file( fname_mod )

			scalefact = 1.0
			offset = 0.
			if( ivar == IVARTYPE_PR ) {
				scalefact = 86400.
				dv      = make_discretevals( 'prism_precip', seq(0.,200,by=20))
				dv_diff = make_discretevals( 'prettyREV', seq(-22, 22, by=4))
				}
			else if( ivar == IVARTYPE_TASMAX ) 
				{
				dv = make_discretevals( 'prism_temp', seq(5,50,by=2.5))
				dv$values2label = seq(5,50,by=5)
				dv_diff = make_discretevals( 'bluered', seq(-5.5, 5.5, by=1))
				offset = -273.15
				}
			else if( ivar == IVARTYPE_TASMIN ) 
				{
				dv = make_discretevals( 'prism_temp', seq(-25,30,by=2.5))
				dv$values2label = seq(5,50,by=5)
				dv_diff = make_discretevals( 'bluered', seq(-5.5, 5.5, by=1))
				offset = -273.15
				}
			else
				{
				#----
				# DTR
				#----
				dv = make_discretevals( 'prism_temp', seq(20,40,by=1))
				dv$values2label = seq(20,40,by=2)
				dv_diff = make_discretevals( 'bluereddesat', seq(-5.5, 5.5, by=1))
				}

			#------------------
			# Scale/offset data
			#------------------
			dat_liv = dat_liv*scalefact + offset
			dat_mod = dat_mod*scalefact + offset

			#------------
			# Plot livneh
			#------------
			ccontour( xvals_liv, yvals_liv, dat_liv,
				discretevals=dv,
				xlim=xlim, ylim=ylim,
				#zrange=zrange,
				contour_show=FALSE,
				cbar_fudge_yoffset=cfyo,
				cbar_fudge_height=cfh,
				raster=TRUE,
				main=paste('Obs', varname, aspect, seas[iseas] ))

			col_map = grey(0.2)
			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )
			mppng_next_frame()

			#-----------
			# Plot model
			#-----------
			ccontour( xvals_mod, yvals_mod, dat_mod,
				discretevals=dv,
				xlim=xlim, ylim=ylim,
				#zrange=zrange,
				contour_show=FALSE,
				cbar_fudge_yoffset=cfyo,
				cbar_fudge_height=cfh,
				raster=TRUE,
				main=paste(modname, varname, aspect, seas[iseas] ))

			col_map = grey(0.2)
			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )
			mppng_next_frame()

			#--------------------------------
			# Compute difference, model - liv
			#--------------------------------
			rv = calc_2d_ratio( xvals_mod, yvals_mod, dat_mod,
					    xvals_liv, yvals_liv, dat_liv,
					    just_difference=TRUE )
			ccontour( rv$xvals, rv$yvals, rv$ratio,
				discretevals=dv_diff,
				xlim=xlim, ylim=ylim,
				contour_show=FALSE,
				cbar_fudge_yoffset=cfyo,
				cbar_fudge_height=cfh,
				raster=TRUE,
				main=paste(modname, '- Obs', aspect, seas[iseas] ))

			col_map = grey(0.2)
			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )

			add_mean( rv$ratio )

			mppng_next_frame()
			}
		}

	return( 1 )
}

#======================================================================================
domodel = function( fname_all, modd, imod, is_last_mod=FALSE, ret_all=NA ) {

	nvars = length( modd$var )

	retval = list()
	for( ivar in 1:nvars ) {
		print(paste(">>>>>>>>>>>> WORKING ON VAR ", ivar, '/', nvars, ':', 
				modd$var[[ivar]]$varname_cmip5, "<<<<<<<<<<"))
		retval[[ivar]] = domodel_var( fname_all, modd, imod, ivar, is_last_mod=is_last_mod, ret_all=ret_all )
		}

	return( retval )
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

	modd = make_mod_descrip( location, LOC_NASA, LOC_CIRRUS )
	nmod = length( modd$model )

	modnames = array(' ', nmod)
	for( imod in 1:nmod )
		modnames[imod] = modd$model[[imod]]$modname_cmip5

	nvars = length(  modd$var )
	varnames = array(' ', nvars)
	for( ivar in 1:nvars ) 
		varnames[ivar] = modd$var[[ivar]]$varname_cmip5

	nexpt = length( modd$expt )
	expt = array(' ', nexpt)
	for( iexpt in 1:nexpt )
		expt[iexpt] = EXPT_NAMES[iexpt]

	period = period_pat
	nperiods = length( period )

	fname_all        = make_all_filenames   ( modd )

	##hierarchy = list( c(modnames,'MMEA'), varnames, expt[2:3], period_pat_label )
	hierarchy = list( c(modnames), varnames )

	#---------------------------------------------------
	# Set up multiple png plots per page in a html bunch
	#---------------------------------------------------
	ppp_x = 3
	ppp_y = 4
	plots_per_page = ppp_x * ppp_y
	npages_tot = (nmod+1) * nvars * 2 * nperiods
	multi_page_png( myname, plots_per_page, npages_tot,
		width=860, height=900,
		header_html='Comparing original vs final model-predicted change',
		hierarchy=hierarchy )

	par(mfrow=c(ppp_y,ppp_x), mar=c(3,1.5,2,0.2), mgp=c(0,0.2,0), oma=c(0,0,3,0) )

	ret_all = list()
	for( imod in 1:nmod ) {	

		print(paste("========================== WORKING ON MOD", imod, "/", nmod, '+++++++++++++++++++' ))
		print(paste("========================== WORKING ON MOD", imod, "/", nmod, '+++++++++++++++++++' ))
		print(paste("========================== WORKING ON MOD", imod, "/", nmod, '+++++++++++++++++++' ))
		print(paste("========================== WORKING ON MOD", imod, "/", nmod, '+++++++++++++++++++' ))
		bn = basename( myname, ".R" )

		ret_all[[imod]] = domodel( fname_all, modd, imod, is_last_mod=(imod==nmod+1), ret_all=ret_all )
		}

	# At this point we have ret_all[[imod]][[ivar]][[iexpt]][[iperiod]][[iseason]][[3]]$x|y|z

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
